@@ -23,14 +23,14 @@
 #include <linux/mutex.h>
 #include <linux/version.h>
 
-#if KERNEL_VERSION(5, 5, 0) <= LINUX_VERSION_CODE
+#if KERNEL_VERSION(5, 5, 0) <= LINUX_VERSION_CODE || defined(EL8)
 #else
 #include <drm/drmP.h>
 #endif
 #include <drm/drm_crtc_helper.h>
 
 #include "evdi_cursor.h"
-#include "evdi_drv.h"
+#include "evdi_drm_drv.h"
 
 /*
  * EVDI drm cursor private structure.
@@ -55,7 +55,11 @@ static void evdi_cursor_set_gem(struct evdi_cursor *cursor,
 	if (obj)
 		drm_gem_object_get(&obj->base);
 	if (cursor->obj)
+#if KERNEL_VERSION(5, 9, 0) <= LINUX_VERSION_CODE
+		drm_gem_object_put(&cursor->obj->base);
+#else
 		drm_gem_object_put_unlocked(&cursor->obj->base);
+#endif
 
 	cursor->obj = obj;
 }
