@@ -10,21 +10,24 @@
 #include "linux/thread_info.h"
 #include "linux/mm.h"
 #include <linux/version.h>
-#if KERNEL_VERSION(5, 5, 0) <= LINUX_VERSION_CODE
+#if KERNEL_VERSION(5, 5, 0) <= LINUX_VERSION_CODE || defined(EL8)
 #else
 #include <drm/drmP.h>
 #endif
 #include <drm/drm_edid.h>
 #include "evdi_drm.h"
-#include "evdi_drv.h"
+#include "evdi_drm_drv.h"
 #include "evdi_cursor.h"
 #include "evdi_params.h"
+#include "evdi_i2c.h"
 #include <linux/mutex.h>
 #include <linux/compiler.h>
+#include <linux/platform_device.h>
+#include <linux/completion.h>
 
 #include <linux/dma-buf.h>
 
-#if KERNEL_VERSION(5, 1, 0) <= LINUX_VERSION_CODE
+#if KERNEL_VERSION(5, 1, 0) <= LINUX_VERSION_CODE || defined(EL8)
 #include <drm/drm_probe_helper.h>
 #endif
 
@@ -58,9 +61,16 @@ struct evdi_event_crtc_state_pending {
 	struct drm_evdi_event_crtc_state crtc_state;
 };
 
+struct evdi_event_ddcci_data_pending {
+	struct drm_pending_event base;
+	struct drm_evdi_event_ddcci_data ddcci_data;
+};
+
 #define MAX_DIRTS 16
 #define EDID_EXT_BLOCK_SIZE 128
 #define MAX_EDID_SIZE (255 * EDID_EXT_BLOCK_SIZE + sizeof(struct edid))
+#define I2C_ADDRESS_DDCCI 0x37
+#define DDCCI_TIMEOUT_MS 50
 
 struct evdi_painter {
 	bool is_connected;
@@ -77,9 +87,15 @@ struct evdi_painter {
 
 	bool was_update_requested;
 	bool needs_full_modeset;
+	struct drm_crtc *crtc;
+	struct drm_pending_vblank_event *vblank;
 
 	struct list_head pending_events;
 	struct delayed_work send_events_work;
+
+	struct completion ddcci_response_received;
+	char *ddcci_buffer;
+	unsigned int ddcci_buffer_length;
 };
 
 static void expand_rect(struct drm_clip_rect *a, const struct drm_clip_rect *b)
@@ -203,11 +219,9 @@ static void copy_cursor_pixels(struct evdi_framebuffer *efb,
 		mutex_unlock(&painter->lock);           \
 	} while (0)
 
-bool evdi_painter_is_connected(struct evdi_device *evdi)
+bool evdi_painter_is_connected(struct evdi_painter *painter)
 {
-	if (evdi && evdi->painter)
-		return evdi->painter->is_connected;
-	return false;
+	return painter ? painter->is_connected : false;
 }
 
 u8 *evdi_painter_get_edid_copy(struct evdi_device *evdi)
@@ -217,7 +231,7 @@ u8 *evdi_painter_get_edid_copy(struct evdi_device *evdi)
 	EVDI_CHECKPT();
 
 	painter_lock(evdi->painter);
-	if (evdi_painter_is_connected(evdi) &&
+	if (evdi_painter_is_connected(evdi->painter) &&
 		evdi->painter->edid &&
 		evdi->painter->edid_length) {
 		block = kmalloc(evdi->painter->edid_length, GFP_KERNEL);
@@ -296,7 +310,7 @@ static bool evdi_painter_flush_pending_events(struct evdi_painter *painter)
 }
 
 static void evdi_painter_send_event(struct evdi_painter *painter,
-				      struct drm_pending_event *event)
+				    struct drm_pending_event *event)
 {
 	if (!event) {
 		EVDI_ERROR("Null drm event!");
@@ -545,9 +559,8 @@ static void evdi_painter_send_mode_changed(
 	evdi_painter_send_event(painter, event);
 }
 
-int evdi_painter_get_num_dirts(struct evdi_device *evdi)
+int evdi_painter_get_num_dirts(struct evdi_painter *painter)
 {
-	struct evdi_painter *painter = evdi->painter;
 	int num_dirts;
 
 	if (painter == NULL) {
@@ -602,8 +615,8 @@ void evdi_painter_mark_dirty(struct evdi_device *evdi,
 		return;
 	}
 
-	painter_lock(evdi->painter);
-	efb = evdi->painter->scanout_fb;
+	painter_lock(painter);
+	efb = painter->scanout_fb;
 	if (!efb) {
 		EVDI_DEBUG("(dev=%d) Skip clip rect. Scanout buffer not set.\n",
 			   evdi->dev_index);
@@ -627,19 +640,67 @@ void evdi_painter_mark_dirty(struct evdi_device *evdi,
 	painter->num_dirts++;
 
 unlock:
-	painter_unlock(evdi->painter);
+	painter_unlock(painter);
 }
 
-void evdi_painter_send_update_ready_if_needed(struct evdi_device *evdi)
+static void evdi_send_vblank(struct drm_crtc *crtc,
+			     struct drm_pending_vblank_event *vblank)
 {
-	struct evdi_painter *painter = evdi->painter;
+	if (crtc && vblank) {
+		unsigned long flags = 0;
+
+		spin_lock_irqsave(&crtc->dev->event_lock, flags);
+		drm_crtc_send_vblank_event(crtc, vblank);
+		spin_unlock_irqrestore(&crtc->dev->event_lock, flags);
+	}
+}
+
+static void evdi_painter_send_vblank(struct evdi_painter *painter)
+{
+	EVDI_CHECKPT();
+
+	evdi_send_vblank(painter->crtc, painter->vblank);
+
+	painter->crtc = NULL;
+	painter->vblank = NULL;
+}
+
+void evdi_painter_set_vblank(
+	struct evdi_painter *painter,
+	struct drm_crtc *crtc,
+	struct drm_pending_vblank_event *vblank)
+{
+	EVDI_CHECKPT();
 
 	if (painter) {
 		painter_lock(painter);
+
+		evdi_painter_send_vblank(painter);
+
+		if (painter->num_dirts > 0 && painter->is_connected) {
+			painter->crtc = crtc;
+			painter->vblank = vblank;
+		} else {
+			evdi_send_vblank(crtc, vblank);
+		}
+
+		painter_unlock(painter);
+	} else {
+		evdi_send_vblank(crtc, vblank);
+	}
+}
+
+void evdi_painter_send_update_ready_if_needed(struct evdi_painter *painter)
+{
+	EVDI_CHECKPT();
+	if (painter) {
+		painter_lock(painter);
+
 		if (painter->was_update_requested) {
 			evdi_painter_send_update_ready(painter);
 			painter->was_update_requested = false;
 		}
+
 		painter_unlock(painter);
 	} else {
 		EVDI_WARN("Painter does not exist!");
@@ -725,6 +786,58 @@ static void evdi_painter_events_cleanup(struct evdi_painter *painter)
 	cancel_delayed_work_sync(&painter->send_events_work);
 }
 
+static void evdi_add_i2c_adapter(struct evdi_device *evdi)
+{
+	struct drm_device *ddev = evdi->ddev;
+	struct platform_device *platdev = to_platform_device(ddev->dev);
+	int result = 0;
+
+	evdi->i2c_adapter = kzalloc(sizeof(*evdi->i2c_adapter), GFP_KERNEL);
+
+	if (!evdi->i2c_adapter) {
+		EVDI_ERROR("(dev=%d) Failed to allocate for i2c adapter",
+			evdi->dev_index);
+		return;
+	}
+
+	result = evdi_i2c_add(evdi->i2c_adapter, &platdev->dev, ddev->dev_private);
+
+	if (result) {
+		kfree(evdi->i2c_adapter);
+		evdi->i2c_adapter = NULL;
+		EVDI_ERROR("(dev=%d) Failed to add i2c adapter, error %d",
+			evdi->dev_index, result);
+		return;
+	}
+
+	EVDI_DEBUG("(dev=%d) Added i2c adapter bus number %d",
+		evdi->dev_index, evdi->i2c_adapter->nr);
+
+	result = sysfs_create_link(&evdi->conn->kdev->kobj,
+			&evdi->i2c_adapter->dev.kobj, "ddc");
+
+	if (result) {
+		EVDI_ERROR("(dev=%d) Failed to create sysfs link, error %d",
+			evdi->dev_index, result);
+		return;
+	}
+}
+
+static void evdi_remove_i2c_adapter(struct evdi_device *evdi)
+{
+	if (evdi->i2c_adapter) {
+		EVDI_DEBUG("(dev=%d) Removing i2c adapter bus number %d",
+			evdi->dev_index, evdi->i2c_adapter->nr);
+
+		sysfs_remove_link(&evdi->conn->kdev->kobj, "ddc");
+
+		evdi_i2c_remove(evdi->i2c_adapter);
+
+		kfree(evdi->i2c_adapter);
+		evdi->i2c_adapter = NULL;
+	}
+}
+
 static int
 evdi_painter_connect(struct evdi_device *evdi,
 		     void const __user *edid_data, unsigned int edid_length,
@@ -783,6 +896,8 @@ evdi_painter_connect(struct evdi_device *evdi,
 	painter->is_connected = true;
 	painter->needs_full_modeset = true;
 
+	evdi_add_i2c_adapter(evdi);
+
 	painter_unlock(painter);
 
 	EVDI_DEBUG("(dev=%d) Connected with %p\n", evdi->dev_index,
@@ -818,8 +933,16 @@ static int evdi_painter_disconnect(struct evdi_device *evdi,
 		   painter->drm_filp);
 	evdi_painter_events_cleanup(painter);
 
+	evdi_painter_send_vblank(painter);
+
 	evdi_cursor_enable(evdi->cursor, false);
 
+	kfree(painter->ddcci_buffer);
+	painter->ddcci_buffer = NULL;
+	painter->ddcci_buffer_length = 0;
+
+	evdi_remove_i2c_adapter(evdi);
+
 	painter->drm_filp = NULL;
 	evdi->dev_index = -1;
 
@@ -828,6 +951,9 @@ static int evdi_painter_disconnect(struct evdi_device *evdi,
 
 	painter_unlock(painter);
 
+	// Signal anything waiting for ddc/ci response with NULL buffer
+	complete(&painter->ddcci_response_received);
+
 	drm_helper_hpd_irq_event(evdi->ddev);
 	return 0;
 }
@@ -880,6 +1006,8 @@ int evdi_painter_grabpix_ioctl(struct drm_device *drm_dev, void *data,
 	struct drm_evdi_grabpix *cmd = data;
 	struct evdi_framebuffer *efb = NULL;
 	struct drm_clip_rect dirty_rects[MAX_DIRTS];
+	struct drm_crtc *crtc = NULL;
+	struct drm_pending_vblank_event *vblank = NULL;
 	int err;
 	int ret;
 	struct dma_buf_attachment *import_attach;
@@ -934,6 +1062,13 @@ int evdi_painter_grabpix_ioctl(struct drm_device *drm_dev, void *data,
 
 	drm_framebuffer_get(&efb->base);
 
+	crtc = painter->crtc;
+	painter->crtc = NULL;
+
+	vblank = painter->vblank;
+	painter->vblank = NULL;
+
+
 	painter_unlock(painter);
 
 	if (!efb->obj->vmapping) {
@@ -990,6 +1125,8 @@ int evdi_painter_grabpix_ioctl(struct drm_device *drm_dev, void *data,
 				       DMA_FROM_DEVICE);
 
 err_fb:
+	evdi_send_vblank(crtc, vblank);
+
 	drm_framebuffer_put(&efb->base);
 
 	return err;
@@ -1049,19 +1186,20 @@ int evdi_painter_init(struct evdi_device *dev)
 		dev->painter->edid = NULL;
 		dev->painter->edid_length = 0;
 		dev->painter->needs_full_modeset = true;
+		dev->painter->crtc = NULL;
+		dev->painter->vblank = NULL;
 		dev->painter->drm_device = dev->ddev;
 		INIT_LIST_HEAD(&dev->painter->pending_events);
 		INIT_DELAYED_WORK(&dev->painter->send_events_work,
 			evdi_send_events_work);
+		init_completion(&dev->painter->ddcci_response_received);
 		return 0;
 	}
 	return -ENOMEM;
 }
 
-void evdi_painter_cleanup(struct evdi_device *evdi)
+void evdi_painter_cleanup(struct evdi_painter *painter)
 {
-	struct evdi_painter *painter = evdi->painter;
-
 	EVDI_CHECKPT();
 	if (!painter) {
 		EVDI_WARN("Painter does not exist\n");
@@ -1073,16 +1211,17 @@ void evdi_painter_cleanup(struct evdi_device *evdi)
 	painter->edid_length = 0;
 	painter->edid = NULL;
 
+	evdi_painter_send_vblank(painter);
+
 	evdi_painter_events_cleanup(painter);
 
 	painter->drm_device = NULL;
 	painter_unlock(painter);
 }
 
-void evdi_painter_set_scanout_buffer(struct evdi_device *evdi,
+void evdi_painter_set_scanout_buffer(struct evdi_painter *painter,
 				     struct evdi_framebuffer *newfb)
 {
-	struct evdi_painter *painter = evdi->painter;
 	struct evdi_framebuffer *oldfb = NULL;
 
 	if (newfb)
@@ -1099,20 +1238,128 @@ void evdi_painter_set_scanout_buffer(struct evdi_device *evdi,
 		drm_framebuffer_put(&oldfb->base);
 }
 
-bool evdi_painter_needs_full_modeset(struct evdi_device *evdi)
+bool evdi_painter_needs_full_modeset(struct evdi_painter *painter)
 {
-	struct evdi_painter *painter = evdi->painter;
-
-	if (painter)
-		return painter->needs_full_modeset;
-	return false;
+	return painter ? painter->needs_full_modeset : false;
 }
 
 
-void evdi_painter_force_full_modeset(struct evdi_device *evdi)
+void evdi_painter_force_full_modeset(struct evdi_painter *painter)
 {
-	struct evdi_painter *painter = evdi->painter;
-
 	if (painter)
 		painter->needs_full_modeset = true;
 }
+
+static struct drm_pending_event *create_ddcci_data_event(struct i2c_msg *msg)
+{
+	struct evdi_event_ddcci_data_pending *event;
+
+	event = kzalloc(sizeof(*event), GFP_KERNEL);
+	if (!event || !msg) {
+		EVDI_ERROR("Failed to create ddcci data event");
+		return NULL;
+	}
+
+	event->ddcci_data.base.type = DRM_EVDI_EVENT_DDCCI_DATA;
+	event->ddcci_data.base.length = sizeof(event->ddcci_data);
+	// Truncate buffers to a maximum of 64 bytes
+	event->ddcci_data.buffer_length = min_t(__u16, msg->len,
+		sizeof(event->ddcci_data.buffer));
+	memcpy(event->ddcci_data.buffer, msg->buf,
+		event->ddcci_data.buffer_length);
+	event->ddcci_data.flags = msg->flags;
+	event->ddcci_data.address = msg->addr;
+
+	event->base.event = &event->ddcci_data.base;
+	return &event->base;
+}
+
+static void evdi_painter_ddcci_data(struct evdi_painter *painter, struct i2c_msg *msg)
+{
+	struct drm_pending_event *event = create_ddcci_data_event(msg);
+
+	reinit_completion(&painter->ddcci_response_received);
+	evdi_painter_send_event(painter, event);
+
+	if (wait_for_completion_interruptible_timeout(
+		&painter->ddcci_response_received,
+		msecs_to_jiffies(DDCCI_TIMEOUT_MS)) > 0) {
+
+		// Match expected buffer length including any truncation
+		const uint32_t expected_response_length = min_t(__u16, msg->len,
+								DDCCI_BUFFER_SIZE);
+
+		painter_lock(painter);
+
+		if (expected_response_length != painter->ddcci_buffer_length)
+			EVDI_WARN("DDCCI buffer length mismatch");
+		else if (painter->ddcci_buffer)
+			memcpy(msg->buf, painter->ddcci_buffer,
+			       painter->ddcci_buffer_length);
+		else
+			EVDI_WARN("Ignoring NULL DDCCI buffer");
+
+		painter_unlock(painter);
+	} else {
+		EVDI_WARN("DDCCI response timeout");
+	}
+}
+
+bool evdi_painter_i2c_data_notify(struct evdi_painter *painter, struct i2c_msg *msg)
+{
+	if (!evdi_painter_is_connected(painter)) {
+		EVDI_WARN("Painter not connected");
+		return false;
+	}
+
+	if (!msg) {
+		EVDI_WARN("Ignored NULL ddc/ci message");
+		return false;
+	}
+
+	if (msg->addr != I2C_ADDRESS_DDCCI) {
+		EVDI_DEBUG("Ignored ddc/ci data for address 0x%x\n", msg->addr);
+		return false;
+	}
+
+	evdi_painter_ddcci_data(painter, msg);
+	return true;
+}
+
+int evdi_painter_ddcci_response_ioctl(struct drm_device *drm_dev, void *data,
+				__always_unused struct drm_file *file)
+{
+	struct evdi_device *evdi = drm_dev->dev_private;
+	struct evdi_painter *painter = evdi->painter;
+	struct drm_evdi_ddcci_response *cmd = data;
+	int result = 0;
+
+	painter_lock(painter);
+
+	// Truncate any read to 64 bytes
+	painter->ddcci_buffer_length = min_t(uint32_t, cmd->buffer_length,
+					     DDCCI_BUFFER_SIZE);
+
+	kfree(painter->ddcci_buffer);
+	painter->ddcci_buffer = kzalloc(painter->ddcci_buffer_length, GFP_KERNEL);
+	if (!painter->ddcci_buffer) {
+		EVDI_ERROR("DDC buffer allocation failed\n");
+		result = -ENOMEM;
+		goto unlock;
+	}
+
+	if (copy_from_user(painter->ddcci_buffer, cmd->buffer,
+		painter->ddcci_buffer_length)) {
+		EVDI_ERROR("Failed to read ddcci_buffer\n");
+		kfree(painter->ddcci_buffer);
+		painter->ddcci_buffer = NULL;
+		result = -EFAULT;
+		goto unlock;
+	}
+
+	complete(&painter->ddcci_response_received);
+
+unlock:
+	painter_unlock(painter);
+	return result;
+}
